@@ -3,7 +3,7 @@
  * ownCloud - files_texteditor
  *
  * @author Tom Needham
- * @copyright 2011 Tom Needham contact@tomneedham.com
+ * @copyright 2013 Tom Needham tom@owncloud.com
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
@@ -19,11 +19,6 @@
  * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
-
-// Init owncloud
- 
-
-
 // Check if we are a user
 OCP\JSON::checkLoggedIn();
 OCP\JSON::callCheck();
@@ -33,40 +28,44 @@ $filecontents = $_POST['filecontents'];
 $path = isset($_POST['path']) ? $_POST['path'] : '';
 $mtime = isset($_POST['mtime']) ? $_POST['mtime'] : '';
 
+$l = OC_L10N::get('files_texteditor');
+
 if($path != '' && $mtime != '') {
 	// Get file mtime
-	$filemtime = OC_Filesystem::filemtime($path);
-	if($mtime != $filemtime)
-	{
+	$filemtime = \OC\Files\Filesystem::filemtime($path);
+	if($mtime != $filemtime) {
 		// Then the file has changed since opening
-		OCP\JSON::error();
-		OCP\Util::writeLog('files_texteditor',"File: ".$path." modified since opening.",OCP\Util::ERROR);	
-	}
-	else
-	{
-		// File same as when opened
-		// Save file
-		if(OC_Filesystem::is_writable($path))
-		{
+		OCP\JSON::error(array('data' => array( 'message' => $l->t('Cannot save file as it has been modified since opening'))));
+		OCP\Util::writeLog(
+			'files_texteditor',
+			"File: ".$path." modified since opening.",
+			OCP\Util::ERROR
+			);
+	} else {
+		// File same as when opened, save file
+		if(\OC\Files\Filesystem::isUpdatable($path)) {
 			$filecontents = iconv(mb_detect_encoding($filecontents), "UTF-8", $filecontents);
-			OC_Filesystem::file_put_contents($path, $filecontents);
+			\OC\Files\Filesystem::file_put_contents($path, $filecontents);
 			// Clear statcache
 			clearstatcache();
 			// Get new mtime
-			$newmtime = OC_Filesystem::filemtime($path);
-			OCP\JSON::success(array('data' => array('mtime' => $newmtime)));
-		}
-		else
-		{
+			$newmtime = \OC\Files\Filesystem::filemtime($path);
+			$newsize = \OC\Files\Filesystem::filesize($path);
+			OCP\JSON::success(array('data' => array('mtime' => $newmtime, 'size' => $newsize)));
+		} else {
 			// Not writeable!
-			OCP\JSON::error(array('data' => array( 'message' => 'Insufficient permissions')));	
-			OCP\Util::writeLog('files_texteditor',"User does not have permission to write to file: ".$path,OCP\Util::ERROR);
+			OCP\JSON::error(array('data' => array( 'message' => $l->t('Insufficient permissions'))));
+			OCP\Util::writeLog(
+				'files_texteditor',
+				"User does not have permission to write to file: ".$path,
+				OCP\Util::ERROR
+				);
 		}
 	}
 } else if($path == '') {
-	OCP\JSON::error(array('data' => array( 'message' => 'File path not supplied')));
+	OCP\JSON::error(array('data' => array( 'message' => $l->t('File path not supplied'))));
 	OCP\Util::writeLog('files_texteditor','No file path supplied', OCP\Util::ERROR);
 } else if($mtime == '') {
-	OCP\JSON::error(array('data' => array( 'message' => 'File mtime not supplied')));
+	OCP\JSON::error(array('data' => array( 'message' => $l->t('File mtime not supplied'))));
 	OCP\Util::writeLog('files_texteditor','No file mtime supplied' ,OCP\Util::ERROR);
 }
