@@ -23,10 +23,10 @@
 
 namespace OCA\user_ldap\lib;
 
-//magic properties (incomplete)
 use OC\ServerNotAvailableException;
 
 /**
+ * magic properties (incomplete)
  * responsible for LDAP connections in context with the provided configuration
  *
  * @property string ldapUserFilter
@@ -34,8 +34,7 @@ use OC\ServerNotAvailableException;
  * @property boolean hasPagedResultSupport
  * @property string[] ldapBaseUsers
  * @property int|string ldapPagingSize holds an integer
- * @property string ldapLoginFilter
- * @property string ldapGroupMemberAssocAttr
+ * @property bool|mixed|void ldapGroupMemberAssocAttr
  */
 class Connection extends LDAPUtility {
 	private $ldapConnectionRes = null;
@@ -514,7 +513,7 @@ class Connection extends LDAPUtility {
 				if(putenv('LDAPTLS_REQCERT=never')) {
 					\OCP\Util::writeLog('user_ldap',
 						'Turned off SSL certificate validation successfully.',
-						\OCP\Util::WARN);
+						\OCP\Util::DEBUG);
 				} else {
 					\OCP\Util::writeLog('user_ldap',
 										'Could not turn off SSL certificate validation.',
