@@ -9,6 +9,7 @@
 namespace OC\Core\Command;
 
 use OC\Updater;
+use OCP\IConfig;
 use Symfony\Component\Console\Command\Command;
 use Symfony\Component\Console\Input\InputInterface;
 use Symfony\Component\Console\Output\OutputInterface;
@@ -23,6 +24,19 @@ class Upgrade extends Command {
 	const ERROR_INVALID_ARGUMENTS = 4;
 	const ERROR_FAILURE = 5;
 
+	/**
+	 * @var IConfig
+	 */
+	private $config;
+
+	/**
+	 * @param IConfig $config
+	 */
+	public function __construct(IConfig $config) {
+		parent::__construct();
+		$this->config = $config;
+	}
+
 	protected function configure() {
 		$this
 			->setName('upgrade')
@@ -38,6 +52,12 @@ class Upgrade extends Command {
 				null,
 				InputOption::VALUE_NONE,
 				'only runs the database schema migration simulation, do not actually update'
+			)
+			->addOption(
+				'--no-app-disable',
+				null,
+				InputOption::VALUE_NONE,
+				'skips the disable of third party apps'
 			);
 	}
 
@@ -49,16 +69,9 @@ class Upgrade extends Command {
 	 */
 	protected function execute(InputInterface $input, OutputInterface $output) {
 
-		require_once \OC::$SERVERROOT . '/lib/base.php';
-
-		// Don't do anything if ownCloud has not been installed
-		if(!\OC_Config::getValue('installed', false)) {
-			$output->writeln('<error>ownCloud has not yet been installed</error>');
-			return self::ERROR_NOT_INSTALLED;
-		}
-
 		$simulateStepEnabled = true;
 		$updateStepEnabled = true;
+		$skip3rdPartyAppsDisable = false;
 
 		if ($input->getOption('skip-migration-test')) {
 			$simulateStepEnabled = false;
@@ -66,6 +79,9 @@ class Upgrade extends Command {
 	   	if ($input->getOption('dry-run')) {
 			$updateStepEnabled = false;
 		}
+		if ($input->getOption('no-app-disable')) {
+			$skip3rdPartyAppsDisable = true;
+		}
 
 		if (!$simulateStepEnabled && !$updateStepEnabled) {
 			$output->writeln(
@@ -76,10 +92,13 @@ class Upgrade extends Command {
 		}
 
 		if(\OC::checkUpgrade(false)) {
-			$updater = new Updater();
+			$self = $this;
+			$updater = new Updater(\OC::$server->getHTTPHelper(),
+				\OC::$server->getConfig());
 
 			$updater->setSimulateStepEnabled($simulateStepEnabled);
 			$updater->setUpdateStepEnabled($updateStepEnabled);
+			$updater->setSkip3rdPartyAppsDisable($skip3rdPartyAppsDisable);
 
 			$updater->listen('\OC\Updater', 'maintenanceStart', function () use($output) {
 				$output->writeln('<info>Turned on maintenance mode</info>');
@@ -102,22 +121,26 @@ class Upgrade extends Command {
 			$updater->listen('\OC\Updater', 'dbSimulateUpgrade', function () use($output) {
 				$output->writeln('<info>Checked database schema update</info>');
 			});
-			$updater->listen('\OC\Updater', 'disabledApps', function ($appList) use($output) {
-				$output->writeln('<info>Disabled incompatible apps: ' . implode(', ', $appList) . '</info>');
+			$updater->listen('\OC\Updater', 'incompatibleAppDisabled', function ($app) use($output) {
+				$output->writeln('<info>Disabled incompatible app: ' . $app . '</info>');
 			});
-			$updater->listen('\OC\Repair', 'info', function ($message) use($output) {
-				$output->writeln('<info>Repair info: ' . $message . '</info>');
+			$updater->listen('\OC\Updater', 'thirdPartyAppDisabled', function ($app) use ($output) {
+				$output->writeln('<info>Disabled 3rd-party app: ' . $app . '</info>');
 			});
-
-			$updater->listen('\OC\Updater', 'failure', function ($message) use($output) {
+			$updater->listen('\OC\Updater', 'repairWarning', function ($app) use($output) {
+				$output->writeln('<error>Repair warning: ' . $app . '</error>');
+			});
+			$updater->listen('\OC\Updater', 'repairError', function ($app) use($output) {
+				$output->writeln('<error>Repair error: ' . $app . '</error>');
+			});
+			$updater->listen('\OC\Updater', 'appUpgradeCheck', function () use ($output) {
+				$output->writeln('<info>Checked database schema update for apps</info>');
+			});
+			$updater->listen('\OC\Updater', 'appUpgrade', function ($app, $version) use ($output) {
+				$output->writeln("<info>Updated <$app> to $version</info>");
+			});
+			$updater->listen('\OC\Updater', 'failure', function ($message) use($output, $self) {
 				$output->writeln("<error>$message</error>");
-				\OC_Config::setValue('maintenance', false);
-			});
-			$updater->listen('\OC\Updater', 'setDebugLogLevel', function ($logLevel, $logLevelName) use($output) {
-				$output->writeln("<info>Set log level to debug - current level: '$logLevelName'</info>");
-			});
-			$updater->listen('\OC\Updater', 'resetLogLevel', function ($logLevel, $logLevelName) use($output) {
-				$output->writeln("<info>Reset log level to '$logLevelName'</info>");
 			});
 
 			$success = $updater->upgrade();
@@ -129,7 +152,7 @@ class Upgrade extends Command {
 			}
 
 			return self::ERROR_SUCCESS;
-		} else if(\OC_Config::getValue('maintenance', false)) {
+		} else if($this->config->getSystemValue('maintenance', false)) {
 			//Possible scenario: ownCloud core is updated but an app failed
 			$output->writeln('<warning>ownCloud is in maintenance mode</warning>');
 			$output->write('<comment>Maybe an upgrade is already in process. Please check the '
@@ -151,7 +174,7 @@ class Upgrade extends Command {
 	 * @param OutputInterface $output output interface
 	 */
 	protected function postUpgradeCheck(InputInterface $input, OutputInterface $output) {
-		$trustedDomains = \OC_Config::getValue('trusted_domains', array());
+		$trustedDomains = $this->config->getSystemValue('trusted_domains', array());
 		if (empty($trustedDomains)) {
 			$output->write(
 				'<warning>The setting "trusted_domains" could not be ' .
