@@ -30,12 +30,10 @@
 
 namespace OC\Files;
 
-use OC\Files\Storage\Loader;
-const SPACE_NOT_COMPUTED = -1;
-const SPACE_UNKNOWN = -2;
-const SPACE_UNLIMITED = -3;
+use OC\Files\Storage\StorageFactory;
 
 class Filesystem {
+
 	/**
 	 * @var Mount\Manager $mounts
 	 */
@@ -47,6 +45,9 @@ class Filesystem {
 	 */
 	static private $defaultInstance;
 
+	static private $usersSetup = array();
+
+	static private $normalizedPathCache = array();
 
 	/**
 	 * classname which used for hooks handling
@@ -166,7 +167,7 @@ class Filesystem {
 	const signal_param_users = 'users';
 
 	/**
-	 * @var \OC\Files\Storage\Loader $loader
+	 * @var \OC\Files\Storage\StorageFactory $loader
 	 */
 	private static $loader;
 
@@ -174,21 +175,30 @@ class Filesystem {
 	 * @param callable $wrapper
 	 */
 	public static function addStorageWrapper($wrapperName, $wrapper) {
-		self::getLoader()->addStorageWrapper($wrapperName, $wrapper);
-
 		$mounts = self::getMountManager()->getAll();
-		foreach ($mounts as $mount) {
-			$mount->wrapStorage($wrapper);
+		if (!self::getLoader()->addStorageWrapper($wrapperName, $wrapper, $mounts)) {
+			// do not re-wrap if storage with this name already existed
+			return;
 		}
 	}
 
+	/**
+	 * Returns the storage factory
+	 *
+	 * @return \OCP\Files\Storage\IStorageFactory
+	 */
 	public static function getLoader() {
 		if (!self::$loader) {
-			self::$loader = new Loader();
+			self::$loader = new StorageFactory();
 		}
 		return self::$loader;
 	}
 
+	/**
+	 * Returns the mount manager
+	 *
+	 * @return \OC\Files\Mount\Manager
+	 */
 	public static function getMountManager() {
 		if (!self::$mounts) {
 			\OC_Util::setupFS();
@@ -251,7 +261,7 @@ class Filesystem {
 
 	/**
 	 * @param string $id
-	 * @return Mount\Mount[]
+	 * @return Mount\MountPoint[]
 	 */
 	public static function getMountByStorageId($id) {
 		if (!self::$mounts) {
@@ -262,7 +272,7 @@ class Filesystem {
 
 	/**
 	 * @param int $id
-	 * @return Mount\Mount[]
+	 * @return Mount\MountPoint[]
 	 */
 	public static function getMountByNumericId($id) {
 		if (!self::$mounts) {
@@ -283,7 +293,7 @@ class Filesystem {
 		}
 		$mount = self::$mounts->find($path);
 		if ($mount) {
-			return array($mount->getStorage(), $mount->getInternalPath($path));
+			return array($mount->getStorage(), rtrim($mount->getInternalPath($path), '/'));
 		} else {
 			return array(null, null);
 		}
@@ -323,7 +333,10 @@ class Filesystem {
 		if ($user == '') {
 			$user = \OC_User::getUser();
 		}
-		$parser = new \OC\ArrayParser();
+		if (isset(self::$usersSetup[$user])) {
+			return;
+		}
+		self::$usersSetup[$user] = true;
 
 		$root = \OC_User::getHome($user);
 
@@ -368,6 +381,11 @@ class Filesystem {
 		self::mountCacheDir($user);
 
 		// Chance to mount for other storages
+		if($userObject) {
+			$mountConfigManager = \OC::$server->getMountProviderCollection();
+			$mounts = $mountConfigManager->getMountsForUser($userObject);
+			array_walk($mounts, array(self::$mounts, 'addMount'));
+		}
 		\OC_Hook::emit('OC_Filesystem', 'post_initMountPoints', array('user' => $user, 'user_dir' => $root));
 	}
 
@@ -429,6 +447,7 @@ class Filesystem {
 	 */
 	public static function clearMounts() {
 		if (self::$mounts) {
+			self::$usersSetup = array();
 			self::$mounts->clear();
 		}
 	}
@@ -444,7 +463,7 @@ class Filesystem {
 		if (!self::$mounts) {
 			\OC_Util::setupFS();
 		}
-		$mount = new Mount\Mount($class, $mountpoint, $arguments, self::getLoader());
+		$mount = new Mount\MountPoint($class, $mountpoint, $arguments, self::getLoader());
 		self::$mounts->addMount($mount);
 	}
 
@@ -494,7 +513,7 @@ class Filesystem {
 		if (!$path || $path[0] !== '/') {
 			$path = '/' . $path;
 		}
-		if (strstr($path, '/../') || strrchr($path, '/') === '/..') {
+		if (strpos($path, '/../') !== FALSE || strrchr($path, '/') === '/..') {
 			return false;
 		}
 		return true;
@@ -525,6 +544,7 @@ class Filesystem {
 	 */
 	static public function isFileBlacklisted($filename) {
 		$filename = self::normalizePath($filename);
+
 		$blacklist = \OC_Config::getValue('blacklisted_files', array('.htaccess'));
 		$filename = strtolower(basename($filename));
 		return in_array($filename, $blacklist);
@@ -679,6 +699,15 @@ class Filesystem {
 		return self::$defaultInstance->searchByMime($query);
 	}
 
+	/**
+	 * @param string|int $tag name or tag id
+	 * @param string $userId owner of the tags
+	 * @return FileInfo[] array or file info
+	 */
+	static public function searchByTag($tag, $userId) {
+		return self::$defaultInstance->searchByTag($tag, $userId);
+	}
+
 	/**
 	 * check if a file or folder has been updated since $time
 	 *
@@ -706,6 +735,12 @@ class Filesystem {
 		 */
 		$path = (string)$path;
 
+		$cacheKey = json_encode([$path, $stripTrailingSlash, $isAbsolutePath]);
+
+		if(isset(self::$normalizedPathCache[$cacheKey])) {
+			return self::$normalizedPathCache[$cacheKey];
+		}
+
 		if ($path == '') {
 			return '/';
 		}
@@ -749,7 +784,10 @@ class Filesystem {
 			$path = substr($path, 0, -2);
 		}
 
-		return $windows_drive_letter . $path;
+		$normalizedPath = $windows_drive_letter . $path;
+		self::$normalizedPathCache[$cacheKey] = $normalizedPath;
+
+		return $normalizedPath;
 	}
 
 	/**
