@@ -39,6 +39,9 @@ class Updater extends BasicEmitter {
 	/** @var bool */
 	private $updateStepEnabled;
 
+	/** @var bool */
+	private $skip3rdPartyAppsDisable;
+
 	/**
 	 * @param HTTPHelper $httpHelper
 	 * @param IConfig $config
@@ -73,6 +76,16 @@ class Updater extends BasicEmitter {
 		$this->updateStepEnabled = $flag;
 	}
 
+	/**
+	 * Sets whether the update disables 3rd party apps.
+	 * This can be set to true to skip the disable.
+	 *
+	 * @param bool $flag false to not disable, true otherwise
+	 */
+	public function setSkip3rdPartyAppsDisable($flag) {
+		$this->skip3rdPartyAppsDisable = $flag;
+	}
+
 	/**
 	 * Check if a new version is available
 	 *
@@ -146,14 +159,20 @@ class Updater extends BasicEmitter {
 		}
 		$this->emit('\OC\Updater', 'maintenanceStart');
 
+		$success = true;
 		try {
 			$this->doUpgrade($currentVersion, $installedVersion);
 		} catch (\Exception $exception) {
-			$this->emit('\OC\Updater', 'failure', array($exception->getMessage()));
+			\OCP\Util::logException('update', $exception);
+			$this->emit('\OC\Updater', 'failure', array(get_class($exception) . ': ' .$exception->getMessage()));
+			$success = false;
 		}
 
 		$this->config->setSystemValue('maintenance', false);
 		$this->emit('\OC\Updater', 'maintenanceEnd');
+		$this->emit('\OC\Updater', 'updateEnd', array($success));
+
+		return $success;
 	}
 
 	/**
@@ -376,9 +395,11 @@ class Updater extends BasicEmitter {
 				continue;
 			}
 
-			// disable any other 3rd party apps
-			\OC_App::disable($app);
-			$this->emit('\OC\Updater', 'thirdPartyAppDisabled', array($app));
+			// disable any other 3rd party apps if not overriden
+			if(!$this->skip3rdPartyAppsDisable) {
+				\OC_App::disable($app);
+				$this->emit('\OC\Updater', 'thirdPartyAppDisabled', array($app));
+			};
 		}
 	}
 
